@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -97,10 +97,6 @@ typedef struct {
 
 } demux_fourxm_t;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_fourxm_class_t;
-
 static float get_le_float(unsigned char *buffer)
 {
   float f;
@@ -121,14 +117,12 @@ static float get_le_float(unsigned char *buffer)
   return f;
 }
 
-/* Open a 4xm file
- * This function is called from the _open() function of this demuxer.
- * It returns 1 if 4xm file was opened successfully. */
-static int open_fourxm_file(demux_fourxm_t *fourxm) {
-  unsigned char preview[12];
+static int probe_fourxm_file(input_plugin_t *input, uint32_t *header_size)
+{
+  unsigned char preview[24];
 
   /* the file signature will be in the first 12 bytes */
-  if (_x_demux_read_header(fourxm->input, preview, 12) != 12)
+  if (_x_demux_read_header(input, preview, 24) != 24)
     return 0;
 
   /* check for the signature tags */
@@ -136,18 +130,23 @@ static int open_fourxm_file(demux_fourxm_t *fourxm) {
       !_x_is_fourcc(&preview[8], "4XMV"))
     return 0;
 
-  /* file is qualified; skip over the header bytes in the stream */
-  fourxm->input->seek(fourxm->input, 12, SEEK_SET);
-
-  /* fetch the LIST-HEAD header */
-  if (fourxm->input->read(fourxm->input, preview, 12) != 12)
-    return 0;
-  if (!_x_is_fourcc(&preview[0], "LIST") ||
-      !_x_is_fourcc(&preview[8], "HEAD") )
+  if (!_x_is_fourcc(&preview[12 + 0], "LIST") ||
+      !_x_is_fourcc(&preview[12 + 8], "HEAD") )
     return 0;
 
-  /* read the whole header */
-  const uint32_t header_size = _X_LE_32(&preview[4]) - 4;
+  *header_size = _X_LE_32(&preview[12 + 4]);
+  if (*header_size < 12)
+    return 0;
+  *header_size -= 4;
+
+  return 1;
+}
+
+/* Open a 4xm file
+ * This function is called from the _open() function of this demuxer.
+ * It returns 1 if 4xm file was opened successfully. */
+static int open_fourxm_file(demux_fourxm_t *fourxm, uint32_t header_size) {
+
   uint8_t *const header = malloc(header_size);
   if (!header || fourxm->input->read(fourxm->input, header, header_size) != header_size) {
     free(header);
@@ -162,7 +161,7 @@ static int open_fourxm_file(demux_fourxm_t *fourxm) {
 
   /* take the lazy approach and search for any and all vtrk and strk chunks */
   int i;
-  for (i = 0; i < header_size - 8; i++) {
+  for (i = 0; i < (int)header_size - 8; i++) {
     const uint32_t fourcc_tag = _X_LE_32(&header[i]);
     const uint32_t size = _X_LE_32(&header[i + 4]);
 
@@ -196,12 +195,13 @@ static int open_fourxm_file(demux_fourxm_t *fourxm) {
           free(header);
           return 0;
         }
-        fourxm->tracks = realloc(fourxm->tracks,
+        void *tmp = realloc(fourxm->tracks,
           fourxm->track_count * sizeof(audio_track_t));
-        if (!fourxm->tracks) {
+        if (!tmp) {
           free(header);
           return 0;
         }
+        fourxm->tracks = tmp;
       }
 
       fourxm->tracks[current_track].channels = _X_LE_32(&header[i + 36]);
@@ -222,11 +222,12 @@ static int open_fourxm_file(demux_fourxm_t *fourxm) {
   /* this will get bumped to 0 on the first iteration */
   fourxm->video_pts = -fourxm->video_pts_inc;
 
-  /* skip the data body LIST header */
-  fourxm->input->seek(fourxm->input, 12, SEEK_CUR);
-
   free(header);
 
+  /* skip the data body LIST header */
+  if (fourxm->input->seek(fourxm->input, 12, SEEK_CUR) < 0)
+    return 0;
+
   return 1;
 }
 
@@ -276,7 +277,7 @@ static int demux_fourxm_send_chunk(demux_plugin_t *this_gen) {
       buf->extra_info->input_time = this->video_pts / 90;
       buf->pts = this->video_pts;
 
-      if (remaining_bytes > buf->max_size)
+      if ((int)remaining_bytes > buf->max_size)
         buf->size = buf->max_size;
       else
         buf->size = remaining_bytes;
@@ -326,7 +327,7 @@ static int demux_fourxm_send_chunk(demux_plugin_t *this_gen) {
       buf->extra_info->input_time = 0;
       buf->pts = 0;
 
-      if (remaining_bytes > buf->max_size)
+      if ((int)remaining_bytes > buf->max_size)
         buf->size = buf->max_size;
       else
         buf->size = remaining_bytes;
@@ -347,7 +348,8 @@ static int demux_fourxm_send_chunk(demux_plugin_t *this_gen) {
 
   case LIST_TAG:
     /* skip LIST header */
-    this->input->seek(this->input, 4, SEEK_CUR);
+    if (this->input->seek(this->input, 4, SEEK_CUR) < 0)
+      this->status = DEMUX_FINISHED;
 
     /* take this opportunity to bump the video pts */
     this->video_pts += this->video_pts_inc;
@@ -420,6 +422,9 @@ static int demux_fourxm_seek (demux_plugin_t *this_gen,
                             off_t start_pos, int start_time, int playing) {
   demux_fourxm_t *this = (demux_fourxm_t *) this_gen;
 
+  (void)start_pos;
+  (void)start_time;
+
   /* if thread is not running, initialize demuxer */
   if( !playing ) {
 
@@ -446,27 +451,55 @@ static int demux_fourxm_get_stream_length (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_fourxm_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_fourxm_get_optional_data(demux_plugin_t *this_gen,
                                         void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
+static void demux_fourxm_dispose(demux_plugin_t *this_gen) {
+
+  demux_fourxm_t *this = (demux_fourxm_t *) this_gen;
+
+  _x_freep (&this->tracks);
+
+  free (this_gen);
+}
+
 static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *stream,
                                     input_plugin_t *input) {
 
   demux_fourxm_t    *this;
+  uint32_t header_size;
+
+  switch (stream->content_detection_method) {
+    case METHOD_BY_MRL:
+    case METHOD_BY_CONTENT:
+    case METHOD_EXPLICIT:
+      if (!probe_fourxm_file(input, &header_size))
+        return NULL;
+      break;
+    default:
+      return NULL;
+  }
+
+  this = calloc(1, sizeof(demux_fourxm_t));
+  if (!this)
+    return NULL;
 
-  this         = calloc(1, sizeof(demux_fourxm_t));
   this->stream = stream;
   this->input  = input;
 
   this->demux_plugin.send_headers      = demux_fourxm_send_headers;
   this->demux_plugin.send_chunk        = demux_fourxm_send_chunk;
   this->demux_plugin.seek              = demux_fourxm_seek;
-  this->demux_plugin.dispose           = default_demux_plugin_dispose;
+  this->demux_plugin.dispose           = demux_fourxm_dispose;
   this->demux_plugin.get_status        = demux_fourxm_get_status;
   this->demux_plugin.get_stream_length = demux_fourxm_get_stream_length;
   this->demux_plugin.get_capabilities  = demux_fourxm_get_capabilities;
@@ -475,38 +508,27 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   this->status = DEMUX_FINISHED;
 
-  switch (stream->content_detection_method) {
-
-  case METHOD_BY_MRL:
-  case METHOD_BY_CONTENT:
-  case METHOD_EXPLICIT:
-
-    if (!open_fourxm_file(this)) {
-      free (this);
-      return NULL;
-    }
-
-  break;
-
-  default:
-    free (this);
+  if (!open_fourxm_file(this, header_size)) {
+    demux_fourxm_dispose (&this->demux_plugin);
     return NULL;
   }
 
   return &this->demux_plugin;
 }
 
-void *demux_fourxm_init_plugin (xine_t *xine, void *data) {
-  demux_fourxm_class_t     *this;
+void *demux_fourxm_init_plugin (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_fourxm_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("4X Technologies (4xm) demux plugin");
-  this->demux_class.identifier      = "4X Technologies";
-  this->demux_class.mimetypes       = NULL;
-  this->demux_class.extensions      = "4xm";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_fourxm_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("4X Technologies (4xm) demux plugin"),
+    .identifier      = "4X Technologies",
+    .mimetypes       = NULL,
+    .extensions      = "4xm",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_fourxm_class;
 }
