@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -75,10 +75,6 @@ typedef struct {
 
 } demux_roq_t ;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_roq_class_t;
-
 /* returns 1 if the RoQ file was opened successfully, 0 otherwise */
 static int open_roq_file(demux_roq_t *this) {
   char preamble[RoQ_CHUNK_PREAMBLE_SIZE];
@@ -154,13 +150,15 @@ static int open_roq_file(demux_roq_t *this) {
     }
 
     /* skip the rest of the chunk */
-    this->input->seek(this->input, chunk_size, SEEK_CUR);
+    if (this->input->seek(this->input, chunk_size, SEEK_CUR) < 0)
+      break;
   }
 
   /* after all is said and done, if there is a width and a height,
    * regard it as being a valid file and reset to the first chunk */
   if (this->bih.biWidth && this->bih.biHeight) {
-    this->input->seek(this->input, 8, SEEK_SET);
+    if (this->input->seek(this->input, 8, SEEK_SET) != 8)
+      return 0;
   } else {
     return 0;
   }
@@ -223,7 +221,7 @@ static int demux_roq_send_chunk(demux_plugin_t *this_gen) {
                                      65535 / this->input->get_length (this->input) );
         buf->pts = audio_pts;
 
-        if (chunk_size > buf->max_size)
+        if ((int)chunk_size > buf->max_size)
           buf->size = buf->max_size;
         else
           buf->size = chunk_size;
@@ -271,7 +269,7 @@ static int demux_roq_send_chunk(demux_plugin_t *this_gen) {
                                      65535 / this->input->get_length (this->input) );
       buf->pts = this->video_pts_counter;
 
-      if (chunk_size > buf->max_size)
+      if ((int)chunk_size > buf->max_size)
         buf->size = buf->max_size;
       else
         buf->size = chunk_size;
@@ -360,6 +358,9 @@ static int demux_roq_seek (demux_plugin_t *this_gen,
 
   demux_roq_t *this = (demux_roq_t *) this_gen;
 
+  (void)start_pos;
+  (void)start_time;
+
   /* if thread is not running, initialize demuxer */
   if( !playing ) {
 
@@ -384,15 +385,20 @@ static int demux_roq_get_status (demux_plugin_t *this_gen) {
 }
 
 static int demux_roq_get_stream_length (demux_plugin_t *this_gen) {
+  (void)this_gen;
   return 0;
 }
 
 static uint32_t demux_roq_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_roq_get_optional_data(demux_plugin_t *this_gen,
 					void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -406,7 +412,10 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
     return NULL;
   }
 
-  this         = calloc(1, sizeof(demux_roq_t));
+  this = calloc(1, sizeof(demux_roq_t));
+  if (!this)
+    return NULL;
+
   this->stream = stream;
   this->input  = input;
 
@@ -443,17 +452,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-void *demux_roq_init_plugin (xine_t *xine, void *data) {
-  demux_roq_class_t     *this;
+void *demux_roq_init_plugin (xine_t *xine, const void *data) {
 
-  this  = calloc(1, sizeof(demux_roq_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("Id RoQ file demux plugin");
-  this->demux_class.identifier      = "RoQ";
-  this->demux_class.mimetypes       = NULL;
-  this->demux_class.extensions      = "roq";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_roq_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("Id RoQ file demux plugin"),
+    .identifier      = "RoQ",
+    .mimetypes       = NULL,
+    .extensions      = "roq",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_roq_class;
 }
