@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -65,31 +65,10 @@
 #endif
 #include "video_out_dxr3.h"
 #include "dxr3.h"
-
-#include "compat.c"
+#include "group_dxr3.h"
 
 #define MAX_SPU_STREAMS 32
 
-
-/* plugin class initialization function */
-static void   *dxr3_spudec_init_plugin(xine_t *xine, void *);
-
-
-/* plugin catalog information */
-static const uint32_t supported_types[] = { BUF_SPU_DVD, 0 };
-
-static const decoder_info_t dxr3_spudec_info = {
-  supported_types,     /* supported types */
-  10                   /* priority        */
-};
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_SPU_DECODER, 17, "dxr3-spudec", XINE_VERSION_CODE, &dxr3_spudec_info, &dxr3_spudec_init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
-
-
 /* plugin class functions */
 static spu_decoder_t *dxr3_spudec_open_plugin(spu_decoder_class_t *class_gen, xine_stream_t *stream);
 
@@ -145,7 +124,6 @@ typedef struct dxr3_spudec_s {
 } dxr3_spudec_t;
 
 /* helper functions */
-static inline int  dxr3_present(xine_stream_t *stream);
 /* the NAV functions must be called with the pci_lock held */
 static inline void dxr3_spudec_clear_nav_list(dxr3_spudec_t *this);
 static inline void dxr3_spudec_update_nav(dxr3_spudec_t *this);
@@ -169,7 +147,7 @@ static inline void dxr3_spudec_update_nav(dxr3_spudec_t *this)
 {
   metronom_clock_t *clock = this->stream->xine->clock;
 
-  if (this->pci_cur.next && this->pci_cur.next->vpts <= clock->get_current_time(clock)) {
+  if (this->pci_cur.next && (int64_t)this->pci_cur.next->vpts <= clock->get_current_time(clock)) {
     pci_node_t *node = this->pci_cur.next;
     xine_fast_memcpy(&this->pci_cur, this->pci_cur.next, sizeof(pci_node_t));
     dxr3_spudec_process_nav(this);
@@ -185,10 +163,13 @@ static inline void dxr3_swab_clut(int *clut)
 }
 
 
-static void *dxr3_spudec_init_plugin(xine_t *xine, void* data)
+void *dxr3_spudec_init_plugin(xine_t *xine, const void* data)
 {
   dxr3_spudec_class_t *this;
 
+  (void)xine;
+  (void)data;
+
   this = calloc(1, sizeof(dxr3_spudec_class_t));
   if (!this) return NULL;
 
@@ -430,7 +411,7 @@ static void dxr3_spudec_decode_data(spu_decoder_t *this_gen, buf_element_t *buf)
 	  case 0x01:  /* show */
 	    /* when only forced SPUs are allowed, change show to hide */
 	    if (spu_channel & 0x80) buf->content[offset_in_buffer] = 0x02;
-	    /* falling through intended */
+	    /* fall through */
 	  case 0x02:  /* hide */
 	    state->parse++;
 	    break;
