@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2020 the xine project
+ * Copyright (C) 2000-2023 the xine project
  * March 2003 - Miguel Freitas
  * This plugin was sponsored by 1Control
  *
@@ -1007,7 +1007,8 @@ static void pvr_event_handler (pvr_input_plugin_t *this) {
         this->input = v4l2_data->input;
 
         /* as of ivtv 0.10.6: must close and reopen to set input */
-        close(this->dev_fd);
+        if (this->dev_fd >= 0)
+          close(this->dev_fd);
         this->dev_fd = xine_open_cloexec(this->devname, O_RDWR);
         if (this->dev_fd < 0) {
           xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
@@ -1046,7 +1047,7 @@ static void pvr_event_handler (pvr_input_plugin_t *this) {
 
         vf.tuner = 0;
         vf.type = vt.type;
-        vf.frequency = (__u32)(freq * fac);
+        vf.frequency = (uint32_t)(freq * fac);
 
         if (ioctl(this->dev_fd, VIDIOC_S_FREQUENCY, &vf) == 0) {
           lprintf("Tuner Frequency set to %d (%f.3 MHz)\n", vf.frequency, vf.frequency / fac);
@@ -1157,7 +1158,8 @@ static void pvr_event_handler (pvr_input_plugin_t *this) {
        pthread_mutex_lock(&this->dev_lock);
 
        /* how lame. we must close and reopen to change bitrate. */
-       close(this->dev_fd);
+       if (this->dev_fd >= 0)
+         close(this->dev_fd);
        this->dev_fd = xine_open_cloexec(this->devname, O_RDWR);
        if (this->dev_fd == -1) {
          pthread_mutex_unlock(&this->dev_lock);
@@ -1600,4 +1602,3 @@ const plugin_info_t xine_plugin_info[] EXPORTED = {
   { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 18, "pvr", XINE_VERSION_CODE, NULL, init_plugin },
   { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
-
