@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 the xine project
+ * Copyright (C) 2003-2020 the xine project
  * Copyright (C) 2003 J.Asselman <j.asselman@itsec-ps.nl>
  *
  * This file is part of xine, a free video player.
@@ -126,11 +126,6 @@ static const int tv_standard_values[] = { VIDEO_MODE_AUTO, VIDEO_MODE_PAL, VIDEO
 
 typedef struct pvrscr_s pvrscr_t;
 
-typedef struct {
-  input_class_t            input_class;
-  xine_t                  *xine;
-} v4l_input_class_t;
-
 typedef struct {
   input_plugin_t           input_plugin;
 
@@ -261,6 +256,7 @@ struct pvrscr_s {
 
 static int pvrscr_get_priority(scr_plugin_t *scr)
 {
+  (void)scr;
   return 10; /* high priority */
 }
 
@@ -523,12 +519,12 @@ static void store_vid_frame (buf_element_t *frame)
   pthread_mutex_unlock (&this->vid_frames_lock);
 }
 
-static int extract_mrl(v4l_input_plugin_t *this, char *mrl)
+static int extract_mrl(v4l_input_plugin_t *this, const char *mrl)
 {
   char   *tuner_name = NULL;
   int     frequency  = 0;
-  char   *locator    = NULL;
-  char   *begin      = NULL;
+  const char   *locator    = NULL;
+  const char   *begin      = NULL;
 
   if (mrl == NULL) {
     lprintf("Someone passed an empty mrl\n");
@@ -1248,6 +1244,9 @@ static int v4l_adjust_realtime_speed(v4l_input_plugin_t *this, fifo_buffer_t *fi
  */
 static off_t v4l_plugin_read (input_plugin_t *this_gen, void *buf, off_t len) {
   lprintf("Read not supported\n");
+  (void)this_gen;
+  (void)buf;
+  (void)len;
   return 0;
 }
 
@@ -1276,6 +1275,9 @@ static buf_element_t *v4l_plugin_read_block (input_plugin_t *this_gen, fifo_buff
   static char           video = 0;
   int                   speed = _x_get_speed(this->stream);
 
+#ifndef LOG
+  (void)todo;
+#endif
   v4l_event_handler(this);
 
 #ifdef HAVE_ALSA
@@ -1303,6 +1305,7 @@ static buf_element_t *v4l_plugin_read_block (input_plugin_t *this_gen, fifo_buff
 
     lprintf("sending video header");
 
+    memset(&bih, 0, sizeof(bih));
     bih.biSize   = sizeof(xine_bmiheader);
     bih.biWidth  = this->resolution->width;
     bih.biHeight = this->resolution->height;
@@ -1452,6 +1455,11 @@ static buf_element_t *v4l_plugin_read_block (input_plugin_t *this_gen, fifo_buff
 static off_t v4l_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin) {
   v4l_input_plugin_t *this = (v4l_input_plugin_t *) this_gen;
 
+  (void)this_gen;
+#ifndef LOG
+  (void)offset;
+  (void)origin;
+#endif
   lprintf("seek %lld bytes, origin %d\n", offset, origin);
   return this->curpos;
 }
@@ -1466,6 +1474,7 @@ static off_t v4l_plugin_get_length (input_plugin_t *this_gen) {
     off_t                 length;
   */
 
+  (void)this_gen;
   return -1;
 }
 
@@ -1489,6 +1498,7 @@ static uint32_t v4l_plugin_get_capabilities (input_plugin_t *this_gen)
  */
 static uint32_t v4l_plugin_get_blocksize (input_plugin_t *this_gen)
 {
+  (void)this_gen;
   return 0;
 }
 
@@ -1556,8 +1566,7 @@ static void v4l_event_handler (v4l_input_plugin_t *this) {
 static void v4l_plugin_dispose (input_plugin_t *this_gen) {
   v4l_input_plugin_t *this = (v4l_input_plugin_t *) this_gen;
 
-  if(this->mrl)
-    free(this->mrl);
+  _x_freep(&this->mrl);
 
   if (this->scr) {
     this->stream->xine->clock->unregister_scr(this->stream->xine->clock, &this->scr->scr);
@@ -1565,8 +1574,7 @@ static void v4l_plugin_dispose (input_plugin_t *this_gen) {
   }
 
   /* Close and free video device */
-  if (this->tuner_name)
-    free(this->tuner_name);
+  _x_freep(&this->tuner_name);
 
   /* Close video device only if device was opened */
   if (this->video_fd > 0) {
@@ -1609,13 +1617,8 @@ static void v4l_plugin_dispose (input_plugin_t *this_gen) {
     snd_pcm_close(this->pcm_handle);
   }
 
-  if (this->pcm_data) {
-    free(this->pcm_data);
-  }
-
-  if (this->pcm_name) {
-    free(this->pcm_name);
-  }
+  _x_freep(&this->pcm_data);
+  _x_freep(&this->pcm_name);
 #endif
 
   if (this->event_queue)
@@ -1627,11 +1630,11 @@ static void v4l_plugin_dispose (input_plugin_t *this_gen) {
      audio_content_base and the other by video_content_base. The
      extra_info structures are all allocated in the first frame
      data. */
-  free(this->audio_content_base);
-  free(this->video_content_base);
+  _x_freep(&this->audio_content_base);
+  _x_freep(&this->video_content_base);
   if (this->frames_base)
-    free(this->frames_base->extra_info);
-  free(this->frames_base);
+    _x_freep(&this->frames_base->extra_info);
+  _x_freep(&this->frames_base);
 
 #ifdef LOG
   printf("\n");
@@ -1657,6 +1660,9 @@ static int v4l_plugin_get_optional_data (input_plugin_t *this_gen,
                                          void *data, int data_type) {
   /* v4l_input_plugin_t *this = (v4l_input_plugin_t *) this_gen; */
 
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
@@ -1725,20 +1731,22 @@ static input_plugin_t *v4l_class_get_instance (input_class_t *cls_gen,
 #ifdef HAVE_ALSA
   cfg_entry_t        *entry;
 #endif
-  char               *mrl     = strdup(data);
 
   /* Example mrl:  v4l:/Television/62500 */
-  if(!mrl || strncasecmp(mrl, "v4l:/", 5)) {
-    free(mrl);
+  if(!data || strncasecmp(data, "v4l:/", 5)) {
     return NULL;
   }
 
   this = calloc(1, sizeof (v4l_input_plugin_t));
-
-  extract_mrl(this, mrl);
+  if (!this)
+    return NULL;
 
   this->stream        = stream;
-  this->mrl           = mrl;
+  this->mrl           = strdup(data);
+  if (!this->mrl) {
+    free(this);
+    return NULL;
+  }
   this->video_buf     = NULL;
   this->video_fd      = -1;
   this->radio_fd      = -1;
@@ -1748,12 +1756,18 @@ static input_plugin_t *v4l_class_get_instance (input_class_t *cls_gen,
   this->pcm_data      = NULL;
   this->pcm_hwparams  = NULL;
 
+  extract_mrl(this, this->mrl);
+
   /* Audio */
   this->pcm_stream    = SND_PCM_STREAM_CAPTURE;
   entry = this->stream->xine->config->lookup_entry(this->stream->xine->config,
                                                    "media.video4linux.audio_device");
   this->pcm_name      = strdup (entry->str_value);
   this->audio_capture = 1;
+  if (!this->pcm_name) {
+    v4l_plugin_dispose(&this->input_plugin);
+    return NULL;
+  }
 #endif
   this->rate          = 44100;
   this->periods       = 2;
@@ -1894,23 +1908,20 @@ static input_plugin_t *v4l_class_get_radio_instance (input_class_t *cls_gen,
 /*
  * v4l input plugin class stuff
  */
-static void *init_video_class (xine_t *xine, void *data)
+static void *init_video_class (xine_t *xine, const void *data)
 {
-  v4l_input_class_t  *this;
+  static const input_class_t v4l_video_input_class = {
+    .get_instance       = v4l_class_get_video_instance,
+    .identifier         = "v4l",
+    .description        = N_("v4l tv input plugin"),
+    .get_dir            = NULL,
+    .get_autoplay_list  = NULL,
+    .dispose            = NULL,
+    .eject_media        = NULL,
+  };
+
   config_values_t    *config = xine->config;
 
-  this = calloc(1, sizeof (v4l_input_class_t));
-
-  this->xine                           = xine;
-
-  this->input_class.get_instance       = v4l_class_get_video_instance;
-  this->input_class.identifier         = "v4l";
-  this->input_class.description        = N_("v4l tv input plugin");
-  this->input_class.get_dir            = NULL;
-  this->input_class.get_autoplay_list  = NULL;
-  this->input_class.dispose            = default_input_class_dispose;
-  this->input_class.eject_media        = NULL;
-
   config->register_filename (config, "media.video4linux.video_device",
 			   VIDEO_DEV, XINE_CONFIG_STRING_IS_DEVICE_NAME,
 			   _("v4l video device"),
@@ -1929,25 +1940,23 @@ static void *init_video_class (xine_t *xine, void *data)
                         _("Selects the TV standard of the input signals. "
                         "Either: AUTO, PAL, NTSC or SECAM. "), 20, NULL, NULL);
 
-  return this;
+  (void)data;
+  return (void *)&v4l_video_input_class;
 }
 
-static void *init_radio_class (xine_t *xine, void *data)
+static void *init_radio_class (xine_t *xine, const void *data)
 {
-  v4l_input_class_t  *this;
-  config_values_t    *config = xine->config;
+  static const input_class_t v4l_radio_input_class = {
+    .get_instance       = v4l_class_get_radio_instance,
+    .identifier         = "v4l",
+    .description        = N_("v4l radio input plugin"),
+    .get_dir            = NULL,
+    .get_autoplay_list  = NULL,
+    .dispose            = NULL,
+    .eject_media        = NULL,
+  };
 
-  this = calloc(1, sizeof (v4l_input_class_t));
-
-  this->xine                           = xine;
-
-  this->input_class.get_instance       = v4l_class_get_radio_instance;
-  this->input_class.identifier         = "v4l";
-  this->input_class.description        = N_("v4l radio input plugin");
-  this->input_class.get_dir            = NULL;
-  this->input_class.get_autoplay_list  = NULL;
-  this->input_class.dispose            = default_input_class_dispose;
-  this->input_class.eject_media        = NULL;
+  config_values_t *config = xine->config;
 
   config->register_filename (config, "media.video4linux.radio_device",
 			   RADIO_DEV, XINE_CONFIG_STRING_IS_DEVICE_NAME,
@@ -1955,7 +1964,8 @@ static void *init_radio_class (xine_t *xine, void *data)
 			   _("The path to your Video4Linux radio device."),
 			   10, NULL, NULL);
 
-  return this;
+  (void)data;
+  return (void *)&v4l_radio_input_class;
 }
 
 /*
@@ -1966,7 +1976,7 @@ const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 18, "v4l_radio", XINE_VERSION_CODE, NULL, init_radio_class },
   { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 18, "v4l_tv", XINE_VERSION_CODE, NULL, init_video_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 
 /*
