@@ -666,7 +666,7 @@ static inline void dump_ImageDescription(void* xxx){
     printf("dataSize=%ld frameCount=%d clutID=%d\n",id->dataSize, id->frameCount, id->clutID);
     printf("name='%.*s'\n",((char*)(&id->name))[0],((char*)(&id->name))+1);
     x=((char*)(&id->clutID))+2;
-    if(id->idSize>sizeof(ImageDescription)){
+    if(id->idSize>(int)sizeof(ImageDescription)){
 	printf("%02X %02X %02X %02X | %02X %02X %02X %02X | %02X %02X %02X %02X | %02X %02X %02X %02X\n",
 	x[0],x[1],x[2],x[3],x[4],x[5],x[6],x[7],x[8],x[9],x[10],x[11],x[12],x[13],x[14],x[15]);
     }
@@ -722,7 +722,7 @@ static inline void dump_CodecDecompressParams(void* xxx){
     idh=cd->imageDescription;
     if(idh && idh[0]) dump_ImageDescription(idh[0]);
 
-    for(i=0;i<sizeof(CodecDecompressParams);i++){
+    for(i=0;i<(int)sizeof(CodecDecompressParams);i++){
 	printf(" %02X",((unsigned char*)cd)[i]);
 	if((i%16)==15) printf("\n");
     }
