@@ -59,13 +59,12 @@ static void copy_scanline( uint8_t *output,
     blit_packed422_scanline( output, data->m1, width );
 }
 
-static int GreedyMaxComb = 15;
-
 static void deinterlace_greedy_packed422_scanline_mmxext( uint8_t *output,
                                                           deinterlace_scanline_data_t *data,
                                                           int width )
 {
-#if defined(ARCH_X86) || defined(ARCH_X86_64)
+#if defined(ARCH_X86)
+    const uint8_t GreedyMaxComb = 15;
     mmx_t MaxComb;
     uint8_t *m0 = data->m0;
     uint8_t *t1 = data->t1;
@@ -173,7 +172,7 @@ static void deinterlace_greedy_packed422_scanline_mmxext( uint8_t *output,
  * work is done in deinterlace_method_t's copy_scanline function.
  */
 
-static deinterlace_method_t greedymethod =
+static const deinterlace_method_t greedymethod =
 {
     "Greedy - Low motion (DScaler)",
     "Greedy",
@@ -198,7 +197,7 @@ static deinterlace_method_t greedymethod =
     "from DScaler."
 };
 
-deinterlace_method_t *greedy_get_method( void )
+const deinterlace_method_t *greedy_get_method( void )
 {
     return &greedymethod;
 }
