@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2011 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -201,7 +201,6 @@ typedef struct {
   video_driver_class_t vo_driver_class;
 
   xine_t *xine;
-  config_values_t *config;
 } pgx64_driver_class_t;
 
 typedef struct {
@@ -1307,6 +1306,7 @@ static void pgx64_dispose(vo_driver_t *this_gen)
   close(this->devfd);
 
   _x_alphablend_free(&this->alphablend_extra_data);
+  _x_vo_scale_cleanup (&this->vo_scale, this->class->xine->config);
 
   free(this);
 }
@@ -1332,13 +1332,14 @@ static void pgx64_config_changed(void *user_data, xine_cfg_entry_t *entry)
  * XINE VIDEO DRIVER CLASS FUNCTIONS
  */
 static const vo_info_t vo_info_pgx64 = {
-  10,
-  XINE_VISUAL_TYPE_X11
+  .priority    = 10,
+  .visual_type = XINE_VISUAL_TYPE_X11,
 };
 
 static vo_driver_t *pgx64_init_driver(video_driver_class_t *class_gen, const void *visual_gen)
 {
   pgx64_driver_class_t *class = (pgx64_driver_class_t *)(void *)class_gen;
+  config_values_t *config = class->xine->config;
   pgx64_driver_t *this;
   struct fbgattr attr;
   long page_size;
@@ -1364,7 +1365,7 @@ static vo_driver_t *pgx64_init_driver(video_driver_class_t *class_gen, const voi
   this->vo_driver.redraw_needed        = pgx64_redraw_needed;
   this->vo_driver.dispose              = pgx64_dispose;
 
-  _x_vo_scale_init(&this->vo_scale, 0, 0, class->config);
+  _x_vo_scale_init(&this->vo_scale, 0, 0, config);
   this->vo_scale.user_ratio      = XINE_VO_ASPECT_AUTO;
   this->vo_scale.user_data       = ((x11_visual_t *)visual_gen)->user_data;
   this->vo_scale.frame_output_cb = ((x11_visual_t *)visual_gen)->frame_output_cb;
@@ -1377,12 +1378,13 @@ static vo_driver_t *pgx64_init_driver(video_driver_class_t *class_gen, const voi
   this->drawable = ((x11_visual_t *)visual_gen)->d;
 
   if (!setup_dga(this)) {
+    _x_vo_scale_cleanup (&this->vo_scale, config);
     free(this);
     return NULL;
   }
 
   if (ioctl(this->devfd, FBIOGATTR, &attr) < 0) {
-    xprintf(this->class->xine, XINE_VERBOSITY_LOG, _("video_out_pgx64: Error: ioctl failed (FBIOGATTR)\n"));
+    xprintf(class->xine, XINE_VERBOSITY_LOG, _("video_out_pgx64: Error: ioctl failed (FBIOGATTR)\n"));
     cleanup_dga(this);
     close(this->devfd);
     free(this);
@@ -1394,7 +1396,7 @@ static vo_driver_t *pgx64_init_driver(video_driver_class_t *class_gen, const voi
   switch (this->fb_type) {
     case FB_TYPE_M64:
       if ((this->vbase = mmap(NULL, (((M64_VRAM_MMAPLEN + page_size - 1) / page_size) * page_size), PROT_READ | PROT_WRITE, MAP_SHARED, this->devfd, 0)) == MAP_FAILED) {
-        xprintf(this->class->xine, XINE_VERBOSITY_DEBUG, "video_out_pgx64: Error: unable to memory map framebuffer\n");
+        xprintf(class->xine, XINE_VERBOSITY_DEBUG, "video_out_pgx64: Error: unable to memory map framebuffer\n");
         cleanup_dga(this);
         close(this->devfd);
         free(this);
@@ -1448,7 +1450,7 @@ static vo_driver_t *pgx64_init_driver(video_driver_class_t *class_gen, const voi
       break;
   }
 
-  this->colour_key  = class->config->register_num(this->class->config, "video.device.pgx64_colour_key", 1,
+  this->colour_key  = config->register_num(config, "video.device.pgx64_colour_key", 1,
     _("video overlay colour key"),
     _("The colour key is used to tell the graphics card where it can overlay the video image. "
       "Try using different values if you see the video showing through other windows."),
@@ -1456,11 +1458,11 @@ static vo_driver_t *pgx64_init_driver(video_driver_class_t *class_gen, const voi
   update_colour_key_rgb(this);
   this->brightness  = 0;
   this->saturation  = 16;
-  this->chromakey_en = class->config->register_bool(this->class->config, "video.device.pgx64_chromakey_en", 0,
+  this->chromakey_en = config->register_bool(config, "video.device.pgx64_chromakey_en", 0,
     _("enable chroma keying"),
     _("Draw OSD graphics on top of the overlay colour key rather than blend them into each frame."),
     20, pgx64_config_changed, this);
-  this->multibuf_en = class->config->register_bool(this->class->config, "video.device.pgx64_multibuf_en", 1,
+  this->multibuf_en = config->register_bool(config, "video.device.pgx64_multibuf_en", 1,
     _("enable multi-buffering"),
     _("Multi buffering increases performance at the expense of using more graphics memory."),
     20, pgx64_config_changed, this);
@@ -1470,7 +1472,7 @@ static vo_driver_t *pgx64_init_driver(video_driver_class_t *class_gen, const voi
   return (vo_driver_t *)this;
 }
 
-static void *pgx64_init_class(xine_t *xine, void *visual_gen)
+static void *pgx64_init_class(xine_t *xine, const void *visual_gen)
 {
   pgx64_driver_class_t *class;
 
@@ -1487,12 +1489,11 @@ static void *pgx64_init_class(xine_t *xine, void *visual_gen)
   class->vo_driver_class.dispose         = default_video_driver_class_dispose;
 
   class->xine   = xine;
-  class->config = xine->config;
 
   return class;
 }
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   {PLUGIN_VIDEO_OUT, 22, "pgx64", XINE_VERSION_CODE, &vo_info_pgx64, pgx64_init_class},
-  {PLUGIN_NONE, 0, "", 0, NULL, NULL}
+  {PLUGIN_NONE, 0, NULL, 0, NULL, NULL}
 };
