@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -147,8 +147,6 @@ struct vidix_driver_s {
 typedef struct vidix_class_s {
   video_driver_class_t driver_class;
 
-  config_values_t     *config;
-
   VDL_HANDLE          vidix_handler;
   vidix_capability_t  vidix_cap;
 
@@ -344,6 +342,9 @@ static uint32_t vidix_get_capabilities (vo_driver_t *this_gen) {
 static void vidixfb_frame_output_cb(void *user_data, int video_width, int video_height, double video_pixel_aspect, int *dest_x, int *dest_y, int *dest_width, int *dest_height, double *dest_pixel_aspect, int *win_x, int *win_y) {
   vidix_driver_t *this = (vidix_driver_t *) user_data;
 
+  (void)video_width;
+  (void)video_height;
+  (void)video_pixel_aspect;
   *dest_x            = 0;
   *dest_y            = 0;
   *dest_width        = this->fb_width;
@@ -356,6 +357,8 @@ static void vidixfb_frame_output_cb(void *user_data, int video_width, int video_
 
 static void vidix_frame_field (vo_frame_t *vo_img, int which_field) {
   /* not needed for vidix */
+  (void)vo_img;
+  (void)which_field;
 }
 
 static void vidix_frame_dispose (vo_frame_t *vo_img) {
@@ -370,6 +373,7 @@ static vo_frame_t *vidix_alloc_frame (vo_driver_t *this_gen) {
   /* vidix_driver_t  *this = (vidix_driver_t *) this_gen; */
   vidix_frame_t   *frame ;
 
+  (void)this_gen;
   frame = (vidix_frame_t *) calloc(1, sizeof(vidix_frame_t));
   if (!frame)
     return NULL;
@@ -406,7 +410,8 @@ static void vidix_compute_ideal_size (vidix_driver_t *this) {
 static void vidix_config_playback (vidix_driver_t *this) {
 
   uint32_t apitch;
-  int err,i;
+  int err;
+  unsigned int i;
 
   _x_vo_scale_compute_output_size( &this->sc );
 
@@ -518,8 +523,9 @@ static void vidix_update_frame_format (vo_driver_t *this_gen,
   vidix_driver_t  *this = (vidix_driver_t *) this_gen;
   vidix_frame_t   *frame = (vidix_frame_t *) frame_gen;
 
-  if ((frame->width != width)
-      || (frame->height != height)
+  (void)flags;
+  if ((frame->width != (int)width)
+      || (frame->height != (int)height)
       || (frame->format != format)) {
 
     /*
@@ -586,6 +592,7 @@ static void vidix_overlay_end (vo_driver_t *this_gen, vo_frame_t *vo_img) {
 #ifdef HAVE_X11
   vidix_driver_t  *this = (vidix_driver_t *) this_gen;
 
+  (void)vo_img;
   if( this->ovl_changed && this->xoverlay ) {
     XLockDisplay (this->display);
     x11osd_expose(this->xoverlay);
@@ -949,13 +956,14 @@ static void vidix_exit (vo_driver_t *this_gen) {
 #endif
 
   _x_alphablend_free(&this->alphablend_extra_data);
+  _x_vo_scale_cleanup (&this->sc, this->config);
 
   free (this);
 }
 
 static vidix_driver_t *open_plugin (video_driver_class_t *class_gen) {
   vidix_class_t        *class = (vidix_class_t *) class_gen;
-  config_values_t      *config = class->config;
+  config_values_t      *config = class->xine->config;
   vidix_driver_t       *this;
   int                   err;
 
@@ -1101,10 +1109,11 @@ static void query_fourccs (vidix_driver_t *this) {
     this->supports_yv12 = 0;
 }
 
-static void *init_class (xine_t *xine, void *visual_gen) {
+static void *init_class (xine_t *xine, const void *visual_gen) {
   vidix_class_t        *this;
   int                   err;
 
+  (void)visual_gen;
   this = (vidix_class_t *) calloc(1, sizeof(vidix_class_t));
   if (!this)
     return NULL;
@@ -1136,7 +1145,6 @@ static void *init_class (xine_t *xine, void *visual_gen) {
 	  _("video_out_vidix: using driver: %s by %s\n"), this->vidix_cap.name, this->vidix_cap.author);
 
   this->xine              = xine;
-  this->config            = xine->config;
 
   return this;
 }
@@ -1148,6 +1156,7 @@ static vo_driver_t *vidix_open_plugin (video_driver_class_t *class_gen, const vo
   const x11_visual_t   *visual = (const x11_visual_t *) visual_gen;
   XWindowAttributes     window_attributes;
 
+  (void)visual_gen;
   this->visual_type       = XINE_VISUAL_TYPE_X11;
 
   this->display           = visual->display;
@@ -1213,7 +1222,7 @@ static vo_driver_t *vidix_open_plugin (video_driver_class_t *class_gen, const vo
   return &this->vo_driver;
 }
 
-static void *vidix_init_class (xine_t *xine, void *visual_gen) {
+static void *vidix_init_class (xine_t *xine, const void *visual_gen) {
 
   vidix_class_t *this = init_class (xine, visual_gen);
 
@@ -1228,8 +1237,8 @@ static void *vidix_init_class (xine_t *xine, void *visual_gen) {
 }
 
 static const vo_info_t vo_info_vidix = {
-  2,                    /* priority    */
-  XINE_VISUAL_TYPE_X11  /* visual type */
+  .priority    = 2,
+  .visual_type = XINE_VISUAL_TYPE_X11,
 };
 #endif
 
@@ -1241,6 +1250,7 @@ static vo_driver_t *vidixfb_open_plugin (video_driver_class_t *class_gen, const
   int                       fd;
   struct fb_var_screeninfo  fb_var;
 
+  (void)visual_gen;
   this->visual_type = XINE_VISUAL_TYPE_FB;
 
   /* Register config option for fb device */
@@ -1287,7 +1297,7 @@ static vo_driver_t *vidixfb_open_plugin (video_driver_class_t *class_gen, const
   return &this->vo_driver;
 }
 
-static void *vidixfb_init_class (xine_t *xine, void *visual_gen) {
+static void *vidixfb_init_class (xine_t *xine, const void *visual_gen) {
 
   vidix_class_t *this = init_class (xine, visual_gen);
 
@@ -1302,8 +1312,8 @@ static void *vidixfb_init_class (xine_t *xine, void *visual_gen) {
 }
 
 static const vo_info_t vo_info_vidixfb = {
-  2,                    /* priority    */
-  XINE_VISUAL_TYPE_FB   /* visual type */
+  .priority    = 2,
+  .visual_type = XINE_VISUAL_TYPE_FB,
 };
 #endif
 
@@ -1319,5 +1329,5 @@ const plugin_info_t xine_plugin_info[] EXPORTED = {
 #ifdef HAVE_FB
   { PLUGIN_VIDEO_OUT, 22, "vidixfb", XINE_VERSION_CODE, &vo_info_vidixfb, vidixfb_init_class },
 #endif
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
