@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2020 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -506,6 +506,7 @@ void xine_set_param (xine_stream_t *s, int param, int value) {
   case XINE_PARAM_VO_CROP_RIGHT:
   case XINE_PARAM_VO_CROP_TOP:
   case XINE_PARAM_VO_CROP_BOTTOM:
+  case XINE_PARAM_VO_TRANSFORM:
     xine->port_ticket->acquire (xine->port_ticket, 1);
     stream->s.video_out->set_property (stream->s.video_out, param, value);
     xine->port_ticket->release (xine->port_ticket, 1);
@@ -701,6 +702,7 @@ int xine_get_param (xine_stream_t *s, int param) {
   case XINE_PARAM_VO_CROP_RIGHT:
   case XINE_PARAM_VO_CROP_TOP:
   case XINE_PARAM_VO_CROP_BOTTOM:
+  case XINE_PARAM_VO_TRANSFORM:
     xine->port_ticket->acquire (xine->port_ticket, 1);
     ret = stream->s.video_out->get_property(stream->s.video_out, param);
     xine->port_ticket->release (xine->port_ticket, 1);
@@ -820,6 +822,84 @@ const char *xine_get_meta_info (xine_stream_t *stream, int info) {
   return _x_meta_info_get_public(stream, info);
 }
 
+int xine_query_stream_info (xine_stream_t *stream, char *sbuf, size_t sblen, int *strings, int *ints) {
+  xine_stream_private_t *_s = (xine_stream_private_t *)stream;
+  static const uint8_t tab_special[sizeof (_s->stream_info) / sizeof (_s->stream_info[0])] = {
+    [XINE_STREAM_INFO_SEEKABLE] = 1,
+    [XINE_STREAM_INFO_HAS_CHAPTERS] = 2,
+    [XINE_STREAM_INFO_MAX_AUDIO_CHANNEL] = 3,
+    [XINE_STREAM_INFO_MAX_SPU_CHANNEL] = 4
+  };
+  int special_index[5] = {-1, -1, -1, -1, -1};
+  uint32_t u, p;
+
+  if (!_s) {
+    if (sbuf && sblen)
+      sbuf[0] = 0;
+    if (strings) {
+      for (u = 0; strings[u] >= 0; u++)
+        strings[0] = 0;
+    }
+    if (ints) {
+      for (u = 0; ints[u] >= 0; u++)
+        ints[0] = 0;
+    }
+    return 0;
+  }
+
+  p = 0;
+  _s = _s->side_streams[0];
+
+  xine_rwlock_rdlock (&_s->info_lock);
+  if (strings) {
+    if (sbuf && sblen)
+      sbuf[p++] = 0;
+    for (u = 0; strings[u] >= 0; u++) {
+      const char *st;
+      if (((uint32_t)strings[u] < sizeof (_s->meta_info) / sizeof (_s->meta_info[0]))
+        && ((st = _s->meta_info[strings[u]]))) {
+        size_t sl = xine_find_byte (st, 0) + 1;
+        if (p + sl > sblen)
+          break;
+        strings[u] = p;
+        memcpy (sbuf + p, st, sl);
+        p += sl;
+      } else {
+        strings[u] = 0;
+      }
+    }
+    for (; strings[u] >= 0; u++)
+      strings[u] = 0;
+  }
+  if (ints) {
+    for (u = 0; ints[u] >= 0; u++) {
+      if ((uint32_t)ints[u] < sizeof (_s->stream_info) / sizeof (_s->stream_info[0])) {
+        special_index[tab_special[ints[u]]] = u;
+        ints[u] = _s->stream_info[ints[u]];
+      } else {
+        ints[u] = 0;
+      }
+    }
+  }
+  xine_rwlock_unlock (&_s->info_lock);
+
+  if (special_index[1] >= 0)
+    ints[special_index[1]] =
+      (_s->s.input_plugin &&
+        (_s->s.input_plugin->get_capabilities (_s->s.input_plugin)
+          & (INPUT_CAP_SEEKABLE | INPUT_CAP_SLOW_SEEKABLE | INPUT_CAP_TIME_SEEKABLE))) ? 1 : 0;
+  if (special_index[2] >= 0)
+    ints[special_index[2]] =
+      (_s->demux.plugin && (_s->demux.plugin->get_capabilities (_s->demux.plugin) & DEMUX_CAP_CHAPTERS)) ||
+      (_s->s.input_plugin && (_s->s.input_plugin->get_capabilities (_s->s.input_plugin) & INPUT_CAP_CHAPTERS));
+  if (special_index[3] >= 0)
+    ints[special_index[3]] = _s->audio_track_map_entries;
+  if (special_index[4] >= 0)
+    ints[special_index[4]] = _s->spu_track_map_entries;
+
+  return p;
+}
+
 xine_osd_t *xine_osd_new(xine_stream_t *stream, int x, int y, int width, int height) {
   xine_osd_t *this = (xine_osd_t *)stream->osd_renderer->new_object(stream->osd_renderer, width, height);
   this->osd.renderer->set_position(&this->osd, x, y);
@@ -1032,6 +1112,8 @@ int _x_message(xine_stream_t *stream, int type, ...) {
     N_("Audio device unavailable"),
     N_("Permission error"),
     N_("File is empty:"),
+    N_("Authentication needed"),
+    N_("Recording done:")
   };
 
   if (!stream) return 0;
