@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020 the xine project
+ *  Copyright (C) 2002-2021 the xine project
  *
  *  This file is part of xine, a free video player.
  *
@@ -522,6 +522,7 @@ int lexer_get_token_d_r(struct lexer * lexer, char ** _tok, int * _tok_size, int
 	  if (strncmp(lexer->lexbuf + lexer->lexbuf_pos, "]]>", 3) == 0) {
 	    lexer->lexbuf_pos += 3;
 	    lexer->lex_mode = DATA;
+	    tok[tok_pos] = '\0';
 	    return T_CDATA_STOP;
 	  } else {
 	    tok[tok_pos] = c;
@@ -542,13 +543,18 @@ int lexer_get_token_d_r(struct lexer * lexer, char ** _tok, int * _tok_size, int
 
     /* pb */
     if (tok_pos + 8 >= tok_size) {
+      char *tmp_tok;
+      int new_size;
       if (fixed)
         return T_ERROR;
-      *_tok_size *= 2;
-      *_tok = realloc (*_tok, *_tok_size);
-      lprintf("token buffer is too small\n");
+      new_size = *_tok_size * 2;
+      tmp_tok = realloc (*_tok, new_size);
+      lprintf("token buffer is too small (need %d)\n", tok_pos);
       lprintf("increasing buffer size to %d bytes\n", *_tok_size);
-      if (*_tok) {
+      if (tmp_tok) {
+	  *_tok = tmp_tok;
+	  memset (*_tok + tok_size, 0, new_size - tok_size);
+	  *_tok_size = new_size;
           return lexer_get_token_d_r (lexer, _tok, _tok_size, 0);
       } else {
           return T_ERROR;
